package com.prodega.proshop.integration.spi;

import static javax.resource.spi.ConnectionEvent.CONNECTION_CLOSED;
import static javax.resource.spi.ConnectionEvent.LOCAL_TRANSACTION_COMMITTED;
import static javax.resource.spi.ConnectionEvent.LOCAL_TRANSACTION_ROLLEDBACK;
import static javax.resource.spi.ConnectionEvent.LOCAL_TRANSACTION_STARTED;

import java.io.PrintWriter;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.security.auth.Subject;
import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

import org.apache.log4j.Logger;

public class GenericManagedConnection implements ManagedConnection, XAResource {

	private final static Logger LOGGER = Logger.getLogger(GenericManagedConnection.class);

	private IOConnectionImpl ioConnection;
	private ConnectionRequestInfo connectionRequestInfo;
	private List<ConnectionEventListener> listeners;
	private int timeout;
	private PrintWriter out;

	private Set<Xid> xids = new HashSet<Xid>();

	GenericManagedConnection(PrintWriter out, ManagedConnectionFactory mcf,
			ConnectionRequestInfo connectionRequestInfo) {
		LOGGER.debug("GenericManagedConnection");
		this.connectionRequestInfo = connectionRequestInfo;
		this.listeners = new LinkedList<ConnectionEventListener>();
		ioConnection = new IOConnectionImpl(out, this, connectionRequestInfo);
	}

	public Object getConnection(Subject subject, ConnectionRequestInfo connectionRequestInfo) {
		LOGGER.debug("GenericManagedConnection.getConnection");
		return ioConnection;
	}

	public void destroy() {
		LOGGER.debug("GenericManagedConnection.destroy");
	}

	public void cleanup() {
		LOGGER.debug("GenericManagedConnection.cleanup");
	}

	public void associateConnection(Object connection) {
		LOGGER.debug("GenericManagedConnection.associateConnection " + connection);
		this.ioConnection = (IOConnectionImpl) connection;

	}

	public void fireConnectionEvent(int event) {
		LOGGER.debug("GenericManagedConnection.fireConnectionEvent");
		ConnectionEvent connnectionEvent = new ConnectionEvent(this, event);
		connnectionEvent.setConnectionHandle(this.ioConnection);
		for (ConnectionEventListener listener : this.listeners) {
			switch (event) {
			case LOCAL_TRANSACTION_STARTED:
				listener.localTransactionStarted(connnectionEvent);
				break;
			case LOCAL_TRANSACTION_COMMITTED:
				listener.localTransactionCommitted(connnectionEvent);
				break;
			case LOCAL_TRANSACTION_ROLLEDBACK:
				listener.localTransactionRolledback(connnectionEvent);
				break;
			case CONNECTION_CLOSED:
				listener.connectionClosed(connnectionEvent);
				break;
			default:
				throw new IllegalArgumentException("Unknown event: " + event);
			}
		}
	}

	public void close() {
		LOGGER.debug("GenericManagedConnection.close");
		this.fireConnectionEvent(CONNECTION_CLOSED);
	}

	public void addConnectionEventListener(ConnectionEventListener listener) {
		LOGGER.debug("GenericManagedConnection.addConnectionEventListener");
		this.listeners.add(listener);
	}

	public void removeConnectionEventListener(ConnectionEventListener listener) {
		LOGGER.debug("GenericManagedConnection.removeConnectionEventListener");
		this.listeners.remove(listener);
	}

	public XAResource getXAResource() {
		LOGGER.debug("GenericManagedConnection.getXAResource");
		return this;
	}

	public LocalTransaction getLocalTransaction() {
		LOGGER.debug("GenericManagedConnection.getLocalTransaction");
		return null;
	}

	public ManagedConnectionMetaData getMetaData() {
		LOGGER.debug("GenericManagedConnection.getMetaData");
		return new ManagedConnectionMetaData() {

			public String getEISProductName() throws ResourceException {
				LOGGER.debug("MyConnectionMetaData.getEISProductName");
				return "FileConnector";
			}

			public String getEISProductVersion() throws ResourceException {
				LOGGER.debug("MyConnectionMetaData.getEISProductVersion");
				return "1.0";
			}

			public int getMaxConnections() throws ResourceException {
				LOGGER.debug("MyConnectionMetaData.getMaxConnections");
				return 5;
			}

			public String getUserName() throws ResourceException {
				return null;
			}
		};
	}

	public void setLogWriter(PrintWriter out) {
		LOGGER.debug("GenericManagedConnection.setLogWriter");
		this.out = out;
	}

	public PrintWriter getLogWriter() {
		LOGGER.debug("GenericManagedConnection.getLogWriter");
		return out;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final GenericManagedConnection other = (GenericManagedConnection) obj;
		if (this.connectionRequestInfo != other.connectionRequestInfo
				&& (this.connectionRequestInfo == null || !this.connectionRequestInfo
						.equals(other.connectionRequestInfo))) {
			return false;
		}
		return true;
	}

	@Override
	public int hashCode() {
		int hash = 5;
		hash = 83 * hash
				+ (this.connectionRequestInfo != null ? this.connectionRequestInfo.hashCode() : 0);
		return hash;
	}

	public ConnectionRequestInfo getConnectionRequestInfo() {
		LOGGER.debug("GenericManagedConnection.getConnectionRequestInfo");
		return connectionRequestInfo;
	}

	@Override
	public void commit(Xid xid, boolean onePhase) throws XAException {
		LOGGER.debug("GenericManagedConnection.commit XA");
		try {
			this.ioConnection.commit();
		} catch (Exception e) {
			throw new XAException(e.getMessage());
		}
	}

	@Override
	public void end(Xid xid, int flags) throws XAException {
		LOGGER.debug("GenericManagedConnection.end XA");
		xids.remove(xid);
	}

	@Override
	public void forget(Xid xid) throws XAException {
		LOGGER.debug("GenericManagedConnection.forget XA");
		this.ioConnection.rollback();
	}

	@Override
	public int getTransactionTimeout() throws XAException {
		LOGGER.debug("GenericManagedConnection.getTransactionTimeout XA");
		return this.timeout;
	}

	@Override
	public boolean isSameRM(XAResource xares) throws XAException {
		LOGGER.debug("GenericManagedConnection.isSameRM XA");
		return this.equals(xares);
	}

	@Override
	public int prepare(Xid xid) throws XAException {
		LOGGER.debug("GenericManagedConnection.prepare XA");
		return XA_OK;
	}

	@Override
	public Xid[] recover(int flag) throws XAException {
		LOGGER.debug("GenericManagedConnection.recover XA");
		return xids.toArray(new Xid[xids.size()]);
	}

	@Override
	public void rollback(Xid xid) throws XAException {
		LOGGER.debug("GenericManagedConnection.rollback XA");
		this.ioConnection.rollback();
	}

	@Override
	public boolean setTransactionTimeout(int seconds) throws XAException {
		LOGGER.debug("GenericManagedConnection.setTransactionTimeout XA");
		this.timeout = seconds;
		return true;
	}

	@Override
	public void start(Xid xid, int flags) throws XAException {
		LOGGER.debug("GenericManagedConnection.start XA");
		xids.add(xid);
		this.ioConnection.initialize();
	}
}
