package com.prodega.proshop.integration.spi;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.UUID;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionRequestInfo;

import org.apache.log4j.Logger;

import com.prodega.proshop.integration.IOConnection;
import com.prodega.proshop.integration.exceptions.IOTransactionException;

public class IOConnectionImpl implements IOConnection {

    private final static Logger LOGGER = Logger.getLogger(IOConnectionImpl.class);

    private TransactionalAction singleTransactionalAction;

    private ConnectionRequestInfo connectionRequestInfo;

    private GenericManagedConnection genericManagedConnection;

    public IOConnectionImpl(PrintWriter out, GenericManagedConnection genericManagedConnection,
            ConnectionRequestInfo connectionRequestInfo) {
        LOGGER.debug("IOConnection " + connectionRequestInfo + " " + toString());
        this.connectionRequestInfo = connectionRequestInfo;
        this.genericManagedConnection = genericManagedConnection;
        this.initialize();
    }

    @Override
    public void write(String filename, byte[] content) {
        LOGGER.debug("IOConnection.write " + filename);
        setSingleTransactionalAction(new WriteAction(filename, content));
    }

    @Override
    public void print(String printer, byte[] content, int count) {
        LOGGER.debug("IOConnection.print " + printer);
        File file = getUniqueTempFile(".pdf");
        if (singleTransactionalAction != null && singleTransactionalAction instanceof PrintAction) {
            PrintAction pa = (PrintAction) singleTransactionalAction;
            try {
                pa.addPdfContent(content, count);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        } else {
            setSingleTransactionalAction(new PrintAction(printer, file.getAbsolutePath(), content, count));
        }
    }

    @Override
    public void runDailyend(boolean closed) throws ResourceException {
        String execString = System.getenv("SH");
        String executable = execString == null ? "sh" : execString;

        LOGGER.debug("IOConnection.runDailyend " + closed);
        try {
            String cmd = executable + " /prodega/scripts/dailyend.sh" + (closed ? " closed" : "");
            Process p = Runtime.getRuntime().exec(cmd);
            int exitCode = p.waitFor();
            if (exitCode != 0) {
                throw new ResourceException("script exited with status " + exitCode);
            }
        } catch (IOException ex) {
            throw new ResourceException(ex);
        } catch (InterruptedException ex) {
            throw new ResourceException(ex);
        }
    }

    @Override
    public void close() {
        this.genericManagedConnection.close();
    }

    public void commit() {
        LOGGER.debug("IOConnection.commit " + toString());
        if (singleTransactionalAction != null) {
            singleTransactionalAction.commit();
        }
    }

    public void rollback() {
        LOGGER.debug("IOConnection.rollback  " + toString());
        if (singleTransactionalAction != null) {
            singleTransactionalAction.rollback();
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final IOConnectionImpl other = (IOConnectionImpl) obj;
        if (this.connectionRequestInfo != other.connectionRequestInfo
                && (this.connectionRequestInfo == null || !this.connectionRequestInfo
                        .equals(other.connectionRequestInfo))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 29 * hash
                + (this.connectionRequestInfo != null ? this.connectionRequestInfo.hashCode() : 0);
        return hash;
    }

    private File getUniqueTempFile(String extension) {
        String tempdir = System.getProperty("java.io.tmpdir");

        if (!(tempdir.endsWith("/") || tempdir.endsWith("\\"))) {
            tempdir = tempdir + System.getProperty("file.separator");
        }
        String fileName = tempdir + UUID.randomUUID().toString() + extension;
        LOGGER.debug("IOConnection.getUniqueTempFile " + fileName);
        return new File(fileName);
    }

    public void initialize() {
        singleTransactionalAction = null;
    }

    private void setSingleTransactionalAction(TransactionalAction action) {
        if (singleTransactionalAction != null) {
            throw new IOTransactionException(
                    "Only one transactional IO action allowed in one transaction");
        }
        this.singleTransactionalAction = action;
    }
}
